﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace WifiAutomation {
    public partial class frmProfile : Form {

        private List<Profile> profiles;
        Dictionary<String, String> actionLabels = Action.actions();
        bool update = true;
        bool addNew;

        internal frmProfile(List<Profile> profiles, bool add) {
            this.addNew = add;
            this.profiles = profiles;
            InitializeComponent();
        }

        internal List<Profile> getProfiles() {
            return profiles;
        }

        private void frmProfile_Load(object sender, EventArgs e) {
            Icon = Properties.Resources.wifi;
            update = false;
            foreach (String action in Action.actions().Values) {
                comboAction.Items.Add(action);
            }
            update = true;
            lstProfiles.DataSource = profiles;
            update = false;
            comboAction.SelectedIndex = 0;
            updateActionControls();
            update = true;
        }

        private void btnNewProfile_Click(object sender, EventArgs e) {
            newEmptyProfile();
        }

        private void newEmptyProfile() {
            profiles.Add(new Profile("<New profile>"));
            updateProfileList();
            lstProfiles.SelectedIndex = lstProfiles.Items.Count - 1;
            selectedProfileChange();
            txtName.SelectAll();
            txtName.Focus();
        }

        private void lstProfiles_SelectedIndexChanged(object sender, EventArgs e) {
            selectedProfileChange();
        }

        private void selectedProfileChange() {
            resetEditBoxes();
            if (lstProfiles.SelectedIndex > -1 && update) {
                grpEdit.Enabled = true;
                txtName.Text = lstProfiles.SelectedItem.ToString();
                lstActions.DataSource = ((Profile)lstProfiles.SelectedItem).actions;
            } else {
                grpEdit.Enabled = false;
            }
            btnDeleteAction.Enabled = lstActions.Items.Count > 0;
        }

        private void resetEditBoxes() {
            update = false;
            lstActions.DataBindings.Clear();
            txtName.Clear();
            resetActionBoxes();
            update = true;
        }

        private void resetActionBoxes() {
            comboAction.SelectedIndex = 0;
            txtPath.Clear();
        }

        private void comboAction_SelectedIndexChanged(object sender, EventArgs e) {
            int i = comboAction.SelectedIndex;
            lblPath.Visible = i == 0 || i == 1;
            txtPath.Visible = i == 0 || i == 1;
            btnBrowse.Visible = i == 0 || i == 1;
            lblVolume.Visible = i == 2;
            numVolume.Visible = i == 2;
            String value = "";
            if (i == 0 || i == 1)
                value = txtPath.Text;
            else if (i == 2)
                value = numVolume.Value.ToString();
            if (lstActions.SelectedIndex > -1 && update) {
                foreach (String key in actionLabels.Keys) {
                    if (actionLabels[key] == comboAction.SelectedItem.ToString()) {
                        ((Action)lstActions.SelectedItem).setAction(key);
                        ((Action)lstActions.SelectedItem).setValue(value);
                        break;
                    }
                }
                updateActionList();
            }
        }

        private void lstActions_SelectedIndexChanged(object sender, EventArgs e) {
            update = false;
            resetActionBoxes();
            updateActionControls();
            update = true;
        }

        private void btnNewAction_Click(object sender, EventArgs e) {
            ((Profile)lstProfiles.SelectedItem).actions.Add(new Action("Run", ""));
            updateActionList();
            lstActions.SelectedIndex = lstActions.Items.Count - 1;
            update = false;
            updateActionControls();
            update = true;
        }

        private void updateActionControls() {
            if (lstActions.SelectedIndex > -1) {
                grpEditAction.Enabled = true;
                Action action = (Action) lstActions.SelectedItem;
                String text = action.getAction();
                String value = action.getValue();
                comboAction.SelectedItem = Action.actions()[text];
                if (text == Action.RUN || text == Action.KILL)
                    txtPath.Text = value;
                else if (text == Action.VOLUME)
                    numVolume.Value = Int32.Parse(value);
            } else {
                grpEditAction.Enabled = false;
                resetActionBoxes();
            }
        }

        private void txtPath_TextChanged(object sender, EventArgs e) {
            if (lstActions.SelectedIndex > -1 && update) {
                String value = txtPath.Text;
                if (comboAction.SelectedIndex == 1) {
                    value = processName(value);
                    if (value.ToLower() == "wifiautomation")
                        MessageBox.Show("Hey, that's not very nice :(");
                }
                ((Action)lstActions.SelectedItem).setValue(value);
                updateActionList();
            }
        }

        private String processName(String value) {
            value.Replace("\"", "");
            int start = Math.Max(value.LastIndexOf('/'), value.LastIndexOf('\\')) + 1;
            int end = value.LastIndexOf('.');
            if (end - start > 0)
                return value.Substring(start, end - start);
            else
                return value.Substring(start);
        }

        private void txtName_TextChanged(object sender, EventArgs e) {
            if (lstProfiles.SelectedIndex > -1 && update) {
                ((Profile)lstProfiles.SelectedItem).setName(txtName.Text);
                updateProfileList();
            }
        }

        private void updateProfileList() {
            ((CurrencyManager)lstProfiles.BindingContext[profiles]).Refresh();
            btnDeleteProfile.Enabled = profiles.Count > 0;
        }

        private void updateActionList() {
            ((CurrencyManager)lstActions.BindingContext[((Profile)lstProfiles.SelectedItem).actions]).Refresh();
            btnDeleteAction.Enabled = lstActions.Items.Count > 0;
        }

        private void btnDeleteProfile_Click(object sender, EventArgs e) {
            deleteSelectedProfile();
        }

        private void lstProfiles_KeyDown(object sender, KeyEventArgs e) {
            if (e.KeyCode == Keys.Delete)
                deleteSelectedProfile();
        }

        private void deleteSelectedProfile() {
            int i = Math.Min(lstProfiles.SelectedIndex, lstProfiles.Items.Count - 2);
            profiles.Remove((Profile) lstProfiles.SelectedItem);
            lstProfiles.SelectedIndex = i;
            updateProfileList();
        }

        private void deleteSelectedAction() {
            int i = Math.Min(lstActions.SelectedIndex, lstActions.Items.Count - 2);
            ((Profile)lstProfiles.SelectedItem).actions.Remove((Action)lstActions.SelectedItem);
            lstActions.SelectedIndex = i;
            updateActionList();
        }

        private void btnDeleteAction_Click(object sender, EventArgs e) {
            deleteSelectedAction();
        }

        private void lstActions_KeyDown(object sender, KeyEventArgs e) {
            deleteSelectedAction();
        }

        private void numVolume_ValueChanged(object sender, EventArgs e) {
            if (lstActions.SelectedIndex > -1 && update) {
                ((Action)lstActions.SelectedItem).setValue(numVolume.Value.ToString());
                updateActionList();
            }
        }

        private void btnBrowse_Click(object sender, EventArgs e) {
            if (openDialog.ShowDialog() == DialogResult.OK) {
                txtPath.Text = openDialog.FileName;
            }
        }
    }
}
